/// <reference types="node" />
import { IlpAddress, isValidIlpAddress } from './utils/address';
import * as errors from './errors';
export declare const Errors: typeof errors;
export { IlpAddress, isValidIlpAddress };
export { getScheme, IlpAddressScheme } from './utils/address';
export interface IlpErrorClass {
    message: string;
    ilpErrorCode?: IlpErrorCode;
    ilpErrorData?: Buffer;
}
export declare enum IlpError {
    F00_BAD_REQUEST = "F00",
    F01_INVALID_PACKET = "F01",
    F02_UNREACHABLE = "F02",
    F03_INVALID_AMOUNT = "F03",
    F04_INSUFFICIENT_DESTINATION_AMOUNT = "F04",
    F05_WRONG_CONDITION = "F05",
    F06_UNEXPECTED_PAYMENT = "F06",
    F07_CANNOT_RECEIVE = "F07",
    F08_AMOUNT_TOO_LARGE = "F08",
    F99_APPLICATION_ERROR = "F99",
    T00_INTERNAL_ERROR = "T00",
    T01_PEER_UNREACHABLE = "T01",
    T02_PEER_BUSY = "T02",
    T03_CONNECTOR_BUSY = "T03",
    T04_INSUFFICIENT_LIQUIDITY = "T04",
    T05_RATE_LIMITED = "T05",
    T99_APPLICATION_ERROR = "T99",
    R00_TRANSFER_TIMED_OUT = "R00",
    R01_INSUFFICIENT_SOURCE_AMOUNT = "R01",
    R02_INSUFFICIENT_TIMEOUT = "R02",
    R99_APPLICATION_ERROR = "R99"
}
export declare type IlpErrorCode = 'F00' | 'F01' | 'F02' | 'F03' | 'F04' | 'F05' | 'F06' | 'F07' | 'F08' | 'F99' | 'T00' | 'T01' | 'T02' | 'T03' | 'T04' | 'T05' | 'T99' | 'R00' | 'R01' | 'R02' | 'R99';
export declare const isCanonicalIlpRejectCode: (o: unknown) => o is IlpErrorCode;
export declare enum Type {
    TYPE_ILP_PREPARE = 12,
    TYPE_ILP_FULFILL = 13,
    TYPE_ILP_REJECT = 14
}
export declare enum IlpPacketType {
    Prepare = 12,
    Fulfill = 13,
    Reject = 14
}
export declare const deserializeEnvelope: (binary: Buffer) => {
    type: number;
    contents: Buffer;
};
export declare type IlpPacket = {
    type: Type.TYPE_ILP_PREPARE;
    typeString?: 'ilp_prepare';
    data: IlpPrepare;
} | {
    type: Type.TYPE_ILP_FULFILL;
    typeString?: 'ilp_fulfill';
    data: IlpFulfill;
} | {
    type: Type.TYPE_ILP_REJECT;
    typeString?: 'ilp_reject';
    data: IlpReject;
};
export interface IlpPrepare {
    amount: string;
    executionCondition: Buffer;
    expiresAt: Date;
    destination: string;
    data: Buffer;
}
export declare const serializeIlpPrepare: (json: IlpPrepare) => Buffer;
export declare const deserializeIlpPrepare: (binary: Buffer) => IlpPrepare;
export interface IlpFulfill {
    fulfillment: Buffer;
    data: Buffer;
}
export declare const isIlpReply: (o: any) => o is IlpReply;
export declare const isPrepare: (o: any) => o is IlpPrepare;
export declare const isFulfill: (o: any) => o is IlpFulfill;
export declare const isReject: (o: any) => o is IlpReject;
export declare const serializeIlpFulfill: (json: IlpFulfill) => Buffer;
export declare const deserializeIlpFulfill: (binary: Buffer) => IlpFulfill;
export interface IlpReject {
    code: string;
    triggeredBy: string;
    message: string;
    data: Buffer;
}
export declare const serializeIlpReject: (json: IlpReject) => Buffer;
export declare const deserializeIlpReject: (binary: Buffer) => IlpReject;
export declare const serializeIlpPacket: (obj: IlpPacket) => Buffer;
export declare const deserializeIlpPacket: (binary: Buffer) => IlpPacket;
export declare type IlpPacketHander = (packet: IlpPrepare) => Promise<IlpReply>;
export declare type IlpAny = IlpPrepare | IlpFulfill | IlpReject;
export declare type IlpReply = IlpFulfill | IlpReject;
export declare const deserializeIlpReply: (data: Buffer) => IlpReply;
export declare const serializeIlpReply: (packet: IlpReply) => Buffer;
export declare const errorToReject: (address: string, error: IlpErrorClass) => Buffer;
export declare const errorToIlpReject: (address: string, error: IlpErrorClass) => IlpReject;
