"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.errorToIlpReject = exports.errorToReject = exports.serializeIlpReply = exports.deserializeIlpReply = exports.deserializeIlpPacket = exports.serializeIlpPacket = exports.deserializeIlpReject = exports.serializeIlpReject = exports.deserializeIlpFulfill = exports.serializeIlpFulfill = exports.isReject = exports.isFulfill = exports.isPrepare = exports.isIlpReply = exports.deserializeIlpPrepare = exports.serializeIlpPrepare = exports.deserializeEnvelope = exports.IlpPacketType = exports.Type = exports.isCanonicalIlpRejectCode = exports.IlpError = exports.isValidIlpAddress = exports.Errors = void 0;
const oer_utils_1 = require("oer-utils");
const date_1 = require("./utils/date");
const assert = require("assert");
const address_1 = require("./utils/address");
Object.defineProperty(exports, "isValidIlpAddress", { enumerable: true, get: function () { return address_1.isValidIlpAddress; } });
const Long = require("long");
const errors = require("./errors");
exports.Errors = errors;
var address_2 = require("./utils/address");
Object.defineProperty(exports, "getScheme", { enumerable: true, get: function () { return address_2.getScheme; } });
var IlpError;
(function (IlpError) {
    IlpError["F00_BAD_REQUEST"] = "F00";
    IlpError["F01_INVALID_PACKET"] = "F01";
    IlpError["F02_UNREACHABLE"] = "F02";
    IlpError["F03_INVALID_AMOUNT"] = "F03";
    IlpError["F04_INSUFFICIENT_DESTINATION_AMOUNT"] = "F04";
    IlpError["F05_WRONG_CONDITION"] = "F05";
    IlpError["F06_UNEXPECTED_PAYMENT"] = "F06";
    IlpError["F07_CANNOT_RECEIVE"] = "F07";
    IlpError["F08_AMOUNT_TOO_LARGE"] = "F08";
    IlpError["F99_APPLICATION_ERROR"] = "F99";
    IlpError["T00_INTERNAL_ERROR"] = "T00";
    IlpError["T01_PEER_UNREACHABLE"] = "T01";
    IlpError["T02_PEER_BUSY"] = "T02";
    IlpError["T03_CONNECTOR_BUSY"] = "T03";
    IlpError["T04_INSUFFICIENT_LIQUIDITY"] = "T04";
    IlpError["T05_RATE_LIMITED"] = "T05";
    IlpError["T99_APPLICATION_ERROR"] = "T99";
    IlpError["R00_TRANSFER_TIMED_OUT"] = "R00";
    IlpError["R01_INSUFFICIENT_SOURCE_AMOUNT"] = "R01";
    IlpError["R02_INSUFFICIENT_TIMEOUT"] = "R02";
    IlpError["R99_APPLICATION_ERROR"] = "R99";
})(IlpError = exports.IlpError || (exports.IlpError = {}));
exports.isCanonicalIlpRejectCode = (o) => typeof o === 'string' && Object.values(IlpError).includes(o);
var Type;
(function (Type) {
    Type[Type["TYPE_ILP_PREPARE"] = 12] = "TYPE_ILP_PREPARE";
    Type[Type["TYPE_ILP_FULFILL"] = 13] = "TYPE_ILP_FULFILL";
    Type[Type["TYPE_ILP_REJECT"] = 14] = "TYPE_ILP_REJECT";
})(Type = exports.Type || (exports.Type = {}));
var IlpPacketType;
(function (IlpPacketType) {
    IlpPacketType[IlpPacketType["Prepare"] = 12] = "Prepare";
    IlpPacketType[IlpPacketType["Fulfill"] = 13] = "Fulfill";
    IlpPacketType[IlpPacketType["Reject"] = 14] = "Reject";
})(IlpPacketType = exports.IlpPacketType || (exports.IlpPacketType = {}));
exports.deserializeEnvelope = (binary) => {
    const envelopeReader = oer_utils_1.Reader.from(binary);
    const type = envelopeReader.readUInt8Number();
    const contents = envelopeReader.readVarOctetString();
    return { type, contents };
};
exports.serializeIlpPrepare = (json) => {
    assert(json.amount && typeof json.amount === 'string', 'amount must be a string');
    assert(Buffer.isBuffer(json.executionCondition) && json.executionCondition.length === 32, 'executionCondition must be a 32-byte buffer');
    assert(json.expiresAt && json.expiresAt instanceof Date, 'expiresAt must be a Date');
    assert(typeof json.destination === 'string', 'destination is required');
    assert(!json.data || Buffer.isBuffer(json.data), 'data must be a buffer');
    const expiresAt = Buffer.from(date_1.dateToInterledgerTime(json.expiresAt), 'ascii');
    const destination = Buffer.from(json.destination, 'ascii');
    const contentSize = 8 +
        date_1.INTERLEDGER_TIME_LENGTH +
        32 +
        oer_utils_1.Predictor.measureVarOctetString(destination.length) +
        oer_utils_1.Predictor.measureVarOctetString(json.data.length);
    const envelopeSize = 1 + oer_utils_1.Predictor.measureVarOctetString(contentSize);
    const envelope = new oer_utils_1.Writer(envelopeSize);
    envelope.writeUInt8(IlpPacketType.Prepare);
    const content = envelope.createVarOctetString(contentSize);
    content.writeUInt64(json.amount);
    content.write(expiresAt);
    content.write(json.executionCondition);
    content.writeVarOctetString(destination);
    content.writeVarOctetString(json.data);
    return envelope.getBuffer();
};
exports.deserializeIlpPrepare = (binary) => {
    const { type, contents } = exports.deserializeEnvelope(binary);
    if (type !== IlpPacketType.Prepare) {
        throw new Error('Packet has incorrect type');
    }
    const reader = oer_utils_1.Reader.from(contents);
    const amount = reader.readUInt64();
    const expiresAt = date_1.interledgerTimeToDate(reader.read(date_1.INTERLEDGER_TIME_LENGTH).toString('ascii'));
    const executionCondition = reader.read(32);
    const destination = reader.readVarOctetString().toString('ascii');
    const data = reader.readVarOctetString();
    return {
        amount,
        executionCondition,
        expiresAt,
        destination,
        data,
    };
};
const isUInt64 = (o) => {
    try {
        const l = Long.fromString(o, true);
        return l.toString() === o;
    }
    catch (_) {
        return false;
    }
};
exports.isIlpReply = (o) => exports.isFulfill(o) || exports.isReject(o);
exports.isPrepare = (o) => typeof o === 'object' &&
    o !== null &&
    typeof o.amount === 'string' &&
    isUInt64(o.amount) &&
    o.expiresAt instanceof Date &&
    !Number.isNaN(o.expiresAt.getTime()) &&
    address_1.isValidIlpAddress(o.destination) &&
    Buffer.isBuffer(o.executionCondition) &&
    o.executionCondition.byteLength === 32 &&
    Buffer.isBuffer(o.data);
exports.isFulfill = (o) => typeof o === 'object' &&
    o !== null &&
    Buffer.isBuffer(o.fulfillment) &&
    o.fulfillment.byteLength === 32 &&
    Buffer.isBuffer(o.data);
exports.isReject = (o) => typeof o === 'object' &&
    o !== null &&
    typeof o.code === 'string' &&
    (address_1.isValidIlpAddress(o.triggeredBy) || o.triggeredBy === '') &&
    typeof o.message === 'string' &&
    Buffer.isBuffer(o.data);
exports.serializeIlpFulfill = (json) => {
    assert(Buffer.isBuffer(json.fulfillment) && json.fulfillment.length === 32, 'fulfillment must be a 32-byte buffer');
    assert(!json.data || Buffer.isBuffer(json.data), 'data must be a buffer');
    const contentSize = 32 + oer_utils_1.Predictor.measureVarOctetString(json.data.length);
    const envelopeSize = 1 + oer_utils_1.Predictor.measureVarOctetString(contentSize);
    const envelope = new oer_utils_1.Writer(envelopeSize);
    envelope.writeUInt8(IlpPacketType.Fulfill);
    const content = envelope.createVarOctetString(contentSize);
    content.write(json.fulfillment);
    content.writeVarOctetString(json.data);
    return envelope.getBuffer();
};
exports.deserializeIlpFulfill = (binary) => {
    const { type, contents } = exports.deserializeEnvelope(binary);
    if (type !== IlpPacketType.Fulfill) {
        throw new Error('Packet has incorrect type');
    }
    const reader = oer_utils_1.Reader.from(contents);
    const fulfillment = reader.read(32);
    const data = reader.readVarOctetString();
    return {
        fulfillment,
        data,
    };
};
const ILP_ERROR_CODE_LENGTH = 3;
const EMPTY_BUFFER = Buffer.alloc(0);
exports.serializeIlpReject = (json) => {
    assert(json.code && typeof json.code === 'string', 'code must be a string');
    assert(typeof json.triggeredBy === 'string', 'triggeredBy must be a string');
    assert(typeof json.message === 'string', 'message must be a string');
    assert(!json.data || Buffer.isBuffer(json.data), 'data must be a buffer');
    const codeBuffer = Buffer.from(json.code, 'ascii');
    if (codeBuffer.length !== ILP_ERROR_CODE_LENGTH) {
        throw new Error('ILP error codes must be three bytes long, received: ' + json.code);
    }
    const triggeredBy = Buffer.from(json.triggeredBy, 'ascii');
    const message = Buffer.from(json.message, 'utf8');
    const data = json.data || EMPTY_BUFFER;
    const contentSize = ILP_ERROR_CODE_LENGTH +
        oer_utils_1.Predictor.measureVarOctetString(triggeredBy.length) +
        oer_utils_1.Predictor.measureVarOctetString(message.length) +
        oer_utils_1.Predictor.measureVarOctetString(data.length);
    const envelopeSize = 1 + oer_utils_1.Predictor.measureVarOctetString(contentSize);
    const envelope = new oer_utils_1.Writer(envelopeSize);
    envelope.writeUInt8(IlpPacketType.Reject);
    const content = envelope.createVarOctetString(contentSize);
    content.write(codeBuffer);
    content.writeVarOctetString(triggeredBy);
    content.writeVarOctetString(message);
    content.writeVarOctetString(data);
    return envelope.getBuffer();
};
exports.deserializeIlpReject = (binary) => {
    const { type, contents } = exports.deserializeEnvelope(binary);
    if (type !== IlpPacketType.Reject) {
        throw new Error('Packet has incorrect type');
    }
    const reader = oer_utils_1.Reader.from(contents);
    const code = reader.read(ILP_ERROR_CODE_LENGTH).toString('ascii');
    const triggeredBy = reader.readVarOctetString().toString('ascii');
    if (!address_1.isValidIlpAddress(triggeredBy) && triggeredBy !== '') {
        throw new Error('Invalid triggeredBy ILP address');
    }
    const message = reader.readVarOctetString().toString('utf8');
    const data = reader.readVarOctetString();
    return {
        code,
        triggeredBy,
        message,
        data,
    };
};
exports.serializeIlpPacket = (obj) => {
    switch (obj.type) {
        case Type.TYPE_ILP_PREPARE:
            return exports.serializeIlpPrepare(obj.data);
        case Type.TYPE_ILP_FULFILL:
            return exports.serializeIlpFulfill(obj.data);
        case Type.TYPE_ILP_REJECT:
            return exports.serializeIlpReject(obj.data);
        default:
            throw new Error('Object has invalid type');
    }
};
exports.deserializeIlpPacket = (binary) => {
    if (binary[0] === Type.TYPE_ILP_PREPARE) {
        return {
            type: binary[0],
            typeString: 'ilp_prepare',
            data: exports.deserializeIlpPrepare(binary),
        };
    }
    else if (binary[0] === Type.TYPE_ILP_FULFILL) {
        return {
            type: binary[0],
            typeString: 'ilp_fulfill',
            data: exports.deserializeIlpFulfill(binary),
        };
    }
    else if (binary[0] === Type.TYPE_ILP_REJECT) {
        return {
            type: binary[0],
            typeString: 'ilp_reject',
            data: exports.deserializeIlpReject(binary),
        };
    }
    else {
        throw new Error('Packet has invalid type');
    }
};
exports.deserializeIlpReply = (data) => data[0] === IlpPacketType.Fulfill ? exports.deserializeIlpFulfill(data) : exports.deserializeIlpReject(data);
exports.serializeIlpReply = (packet) => exports.isFulfill(packet) ? exports.serializeIlpFulfill(packet) : exports.serializeIlpReject(packet);
exports.errorToReject = (address, error) => {
    return exports.serializeIlpReject({
        code: error.ilpErrorCode || 'F00',
        triggeredBy: address,
        message: error.message || '',
        data: error.ilpErrorData || Buffer.alloc(0),
    });
};
exports.errorToIlpReject = (address, error) => {
    return {
        code: error.ilpErrorCode || 'F00',
        triggeredBy: address,
        message: error.message || '',
        data: error.ilpErrorData || Buffer.alloc(0),
    };
};
//# sourceMappingURL=index.js.map